//>	AtmosSocket
//
// This class is a wrapper around the Atmosphere library, that provides a simpler
// API to subscribe and push messages to an Atmosphere channel.
// It is a similar API to the one provided in the GWT counterpart of this example.
//
isc.defineClass("AtmosSocket", Class);

isc.AtmosSocket.addProperties({
	
	// The context set when we configured the AtmosphereServlet
	// in web.xml deployment descriptor, as "url-pattern"
	context: "jersey",
	
	// The identifier of the Jersey service as set with the @Path annotation
	service: "",
	
	// Channel identifier, used to create a one-to-one communications channel
	// between the server and this client
	channelId: "",
	
	// Transport to use during communications. If a transport method is
	// not available on server, the protocol will find the best protocol to fallback
	transport: "websocket",
	
	// Preferred fallback transport to use, in case of transport method is
	// not available on server
    fallbackTransport: "long-polling",
    
    logLevel : "info",
    
    // function that will be called when a new message arrives
    messageHandler: function (response) {},

	// function that will be called when the subscription has been done
    // We should not use the pushData() function before the subscription has been done
    subscribedHandler: function (response) {},
    
    request: {},
    
    subSocket: {}
});

isc.AtmosSocket.addMethods({
	// call this function to open the channel 
	// to send and receive messages to/from the server
	subscribe : function() {
		atmosphere.unsubscribe();
		
		this.request = {
			url: document.location.href + this.context + "/" + this.service + '/' + this.channelId,
			contentType : "application/json",
			logLevel : this.logLevel,
			transport : this.transport ,
			fallbackTransport: this.fallbackTransport,
			// This is what we will do when a message is received
			onMessage: this.messageHandler,
			onOpen: this.subscribedHandler,
			onError: function(response) {
				console.log("Sorry, but there is some problem with your " +
					"socket or the server is down. Response: " + response);
	 		}
		};
		
		this.subSocket = atmosphere.subscribe(this.request);
	},
	
	// call this function to send a message to the server
	pushData : function(data) {
		this.subSocket.push(data);
	},

	unsubscribe : function() {
		atmosphere.unsubscribe();
	}
});

isc.AtmosSocket.addClassMethods({
	subscribe : function(service, channelId, messageHandler, subscribedHandler) {
		var atmSocket = isc.AtmosSocket.create({
			service: service,
			channelId: channelId,
			messageHandler: messageHandler,
			subscribedHandler: subscribedHandler
		});
		atmSocket.subscribe();
		return atmSocket;
	}
});
